"""Implement an exception class for the Hut"""

class HutError(Exception):
    """Custom exceptions class for the `Hut` and its subclasses."""
    def __init__(self, message=''):
        super().__init__(message)
        self.padding = '~'*50 + '\n'
        
class HutNumberGreaterThanFiveError(HutError):
    """Report error when user type a hut number greater than 5."""
    def __init__(self, message=''):
        super().__init__(message)
        self.error_message = (self.padding +
                              "ERROR: Number should be less than 5." +
                              '\n' + self.padding)

class NegativeHutNumberError(HutError):
    """Report error when user type a negative hut number."""
    def __init__(self, message=''):
        super().__init__(message)
        self.error_message = (self.padding +
                              "ERROR: Number should be greater than 0." +
                              '\n' + self.padding)

class NotIntegerHutNumberError(HutError):
    """Report error when user type a hut number that is not an integer."""
    def __init__(self, message=''):
        super().__init__(message)
        self.error_message = (self.padding +
                              "ERROR: Number should be in the range 1-5." +
                              '\n' + self.padding)
